from contextlib import contextmanager
from docker.types import DeviceRequest
from logging import getLogger


LOGGER = getLogger(__name__)


@contextmanager
def create_temporary_docker_volume(client, *args, **kwargs):
    volume = client.volumes.create(*args, **kwargs)
    LOGGER.debug(f'Created temporary Docker volume {volume.short_id}')
    try:
        yield volume
    finally:
        volume.remove(force=True)
        LOGGER.debug(f'Deleted temporary Docker volume {volume.short_id}')


@contextmanager
def create_temporary_docker_container(client, *args, gpu: bool = False, **kwargs):
    if gpu:
        kwargs = {**kwargs, **{'device_requests': [DeviceRequest(count=-1, capabilities=[['gpu']])]}}
    container = client.containers.create(*args, **kwargs)
    LOGGER.debug(f'Created temporary Docker container {container.short_id}')
    try:
        yield container
    finally:
        container.remove(force=True)
        LOGGER.debug(f'Deleted temporary Docker container {container.short_id}')
